package cn.com.qmth.stmms.admin.exam;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.query.MarkerSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkTrackService;
import cn.com.qmth.stmms.biz.mark.service.TaskService;
import cn.com.qmth.stmms.common.auth.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

@Controller("examMarkerController")
@RequestMapping("/admin/exam/marker")
public class MarkerController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(MarkerController.class);

    public static final String USER_PASSWORD = "123456";

    @Autowired
    private MarkerService markerService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private MarkTrackService trackService;

    @Autowired
    private TaskService taskService;

    @RequestMapping
    public String list(Model model, HttpServletRequest request, MarkerSearchQuery query) {
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query.orderById();
        subjectFilter(query, wu);
        query = markerService.findByQuery(query);
        for (Marker marker : query.getResult()) {
            marker.setSubject(subjectService.find(marker.getExamId(), marker.getSubjectCode()));
            marker.setGroup(groupService.findOne(examId, marker.getSubjectCode(), marker.getGroupNumber()));
            marker.setMarkedCount(libraryService.countByMarker(marker.getId()));
            marker.setCurrentCount(taskService.countCurrent(marker));
        }
        model.addAttribute("query", query);
        model.addAttribute("subjectList", getExamSubject(examId, wu));
        return "modules/exam/markerList";
    }

    @RequestMapping("/query")
    @ResponseBody
    public JSONArray query(HttpServletRequest request, @RequestParam String subjectCode,
            @RequestParam Integer groupNumber) {
        int examId = getSessionExamId(request);
        JSONArray array = new JSONArray();
        List<Marker> list = markerService.findByExamAndSubjectAndGroup(examId, subjectCode, groupNumber);
        for (Marker marker : list) {
            JSONObject obj = new JSONObject();
            obj.accumulate("id", marker.getId());
            obj.accumulate("loginName", marker.getLoginName());
            obj.accumulate("name", marker.getName());
            array.add(obj);
        }
        return array;
    }

    @RequestMapping("/release")
    @ResponseBody
    @RoleRequire(Role.SCHOOL_ADMIN)
    public JSONObject release(@RequestParam Integer id) {
        JSONObject result = new JSONObject();
        Marker marker = markerService.findById(id);
        if (marker != null) {
            taskService.clearCurrent(marker);
            result.accumulate("success", true);
        } else {
            result.accumulate("success", false);
            result.accumulate("message", "评卷员不存在");
        }
        return result;
    }

    @RequestMapping("/batch-create")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String createInit(Model model, HttpServletRequest request,
            @RequestParam(required = false) String subjectCode, @RequestParam(required = false) Integer groupNumber) {
        int examId = getSessionExamId(request);
        model.addAttribute("subjectList", getExamSubject(examId, RequestUtils.getWebUser(request)));
        if (subjectCode != null) {
            model.addAttribute("subjectCode", subjectCode);
            model.addAttribute("groupList", groupService.findByExamAndSubject(examId, subjectCode));
            if (groupNumber != null) {
                model.addAttribute("groupNumber", groupNumber);
            }
        } else {
            model.addAttribute("groupList", new ArrayList<MarkGroup>());
        }
        return "modules/exam/markerCreate";
    }

    @RequestMapping(value = "/batch-create", method = RequestMethod.POST)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public ModelAndView create(HttpServletRequest request, @RequestParam String subjectCode,
            @RequestParam Integer groupNumber, @RequestParam Integer count, @RequestParam String password) {
        int examId = getSessionExamId(request);
        ExamSubject subject = subjectService.find(examId, subjectCode);
        ModelAndView view = new ModelAndView("redirect:/admin/exam/marker");
        if (subject != null && count > 0 && StringUtils.isNotBlank(password)) {
            markerService.batchCreate(subject, groupNumber, count, password);
        }
        view.addObject("subjectCode", subjectCode);
        if (groupNumber > 0) {
            view.addObject("groupNumber", groupNumber);
        }
        return view;
    }

    @RequestMapping(value = "/delete")
    @Transactional
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String delete(@RequestParam Integer id, Model model, RedirectAttributes redirectAttributes) {
        Marker marker = markerService.findById(id);
        if (marker != null && !isMarking(marker.getExamId(), marker.getSubjectCode())) {
            libraryService.resetByMarker(marker);
            taskService.clearCurrent(marker);
            groupService.updateLibraryCount(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
            trackService.deleteByMarkerId(id);
            markerService.deleteById(id);
        }
        redirectAttributes.addAttribute("subjectCode", marker.getSubjectCode());
        redirectAttributes.addAttribute("groupNumber", marker.getGroupNumber());
        return "redirect:/admin/exam/marker";
    }

    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    @ResponseBody
    @Transactional
    @RoleRequire(Role.SCHOOL_ADMIN)
    public JSONObject deleteMarker(@RequestParam Integer id) {
        Marker marker = markerService.findById(id);
        JSONObject obj = new JSONObject();
        if (marker != null) {
            libraryService.resetByMarker(marker);
            taskService.clearCurrent(marker);
            groupService.updateLibraryCount(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
            trackService.deleteByMarkerId(id);
            markerService.deleteById(id);
            obj.accumulate("success", true);
        } else {
            obj.accumulate("success", false);
            obj.accumulate("message", "该评卷员不存在");
        }
        return obj;
    }

    @RequestMapping(value = "/toggle", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire(Role.SCHOOL_ADMIN)
    public JSONObject toggle(@RequestParam Integer id, @RequestParam Boolean enable) {
        Marker marker = markerService.findById(id);
        JSONObject obj = new JSONObject();
        if (marker != null) {
            marker.setEnable(enable);
            markerService.save(marker);
            obj.accumulate("success", true);
        } else {
            obj.accumulate("success", false);
            obj.accumulate("message", "该评卷员不存在");
        }
        return obj;
    }

    @RequestMapping(value = "/export", method = RequestMethod.POST)
    public String exportFile(MarkerSearchQuery query, HttpServletRequest request, HttpServletResponse response,
            RedirectAttributes redirectAttributes) {
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        try {
            query.setPageNumber(1);
            query.setPageSize(Integer.MAX_VALUE);
            query.setExamId(examId);
            query.orderById();
            subjectFilter(query, wu);
            String fileName = "评卷员账号.xlsx";
            query = markerService.findByQuery(query);
            for (Marker marker : query.getResult()) {
                ExamSubject subject = subjectService.find(examId, marker.getSubjectCode());
                if (subject != null) {
                    marker.setSubjectName(subject.getCode() + "-" + subject.getName());
                } else {
                    marker.setSubjectName("");
                }
                MarkGroup group = groupService.findOne(examId, marker.getSubjectCode(), marker.getGroupNumber());
                if (group != null) {
                    marker.setGroupName(group.getNumber() + "-" + group.getTitle());
                } else {
                    marker.setGroupName("");
                }
            }
            new ExportExcel("评卷员数据", Marker.class).setDataList(query.getResult()).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导出评卷员失败！" + e.getMessage());
        }
        return "redirect:/admin/exam/marker?repage";
    }

}
